@@ -1,7 +1,7 @@
 /*
  * Constant.java
  *
- * Copyright (c) 2006 Operational Dynamics Consulting Pty Ltd
+ * Copyright (c) 2006-2008 Operational Dynamics Consulting Pty Ltd
  * 
  * The code in this file, and the library it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
@@ -52,8 +52,8 @@ public abstract class Constant
      */
     /*
      * In GTK, enums are frequently registered as <code>GEnumValues</code>
-     * with <code>g_enum_register_static()</code>. This, however, is not
-     * done consistently, and cannot be relied upon.
+     * with <code>g_enum_register_static()</code>. This, however, is not done
+     * consistently, and cannot be relied upon.
      */
     final String nickname;
 
@@ -81,6 +81,6 @@ public abstract class Constant
      * @since 4.0.1
      */
     public String toString() {
-        return this.getClass().getName() + "." + nickname;
+        return this.getClass().getSimpleName() + "." + nickname;
     }
 }
