@@ -1,16 +1,17 @@
 /*
  * Plumbing.java
  *
- * Copyright (c) 2007 Operational Dynamics Consulting Pty Ltd
+ * Copyright (c) 2007-2009 Operational Dynamics Consulting Pty Ltd
  * 
  * The code in this file, and the library it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
- * version 2" See the LICENCE file for the terms governing usage and
- * redistribution.
+ * version 2" plus the "Classpath Exception" (you may link to this code as a
+ * library into other programs provided you don't make a derivation of it).
+ * See the LICENCE file for the terms governing usage and redistribution.
+
  */
 package org.gnome.gdk;
 
-import org.freedesktop.bindings.Proxy;
 import org.gnome.glib.Boxed;
 
 /**
@@ -23,29 +24,31 @@ public abstract class Plumbing extends org.gnome.glib.Plumbing
 {
     protected Plumbing() {}
 
+    static {
+        /*
+         * When you call getScreen(), the object returned appears to be a
+         * sublcass of GdkScreen, in this case GdkScreenX11. It doesn't seem
+         * to be public, thus making GdkScreen the "concrete interface" to it
+         * anyway. This actually reflects the fact that we don't have an
+         * architectural treatment for dealing with getting GObject instances
+         * back that are subtypes we don't know about.
+         */
+        registerType("GdkScreenX11", org.gnome.gdk.Screen.class);
+    }
+
     /**
      * GdkEvent is a union, which would ordinarily present an insurmountable
      * problem, but since the GdkEvent structures all begin with a
      * GdkEventType we write a very hacky override which uses that as a
      * discriminator.
      */
-    protected static Boxed boxedFor(Class type, long pointer) {
-        Proxy proxy;
+    protected static Boxed boxedFor(Class<? extends Boxed> type, long pointer) {
+        Boxed proxy;
 
         if (pointer == 0L) {
             return null;
         }
 
-        proxy = instanceFor(pointer);
-
-        if (proxy != null) {
-            /*
-             * A Proxy already exists for this. Somewhat unexpected, but good
-             * all the same. Simply return it.
-             */
-            return (Boxed) proxy;
-        }
-
         /*
          * Handle the special case of the GdkEvent union.
          */
@@ -55,6 +58,8 @@ public abstract class Plumbing extends org.gnome.glib.Plumbing
 
             // FIXME this must be wrong, but what else should we do with it?
             case GdkEventType.DELETE:
+            case GdkEventType.MAP:
+            case GdkEventType.UNMAP:
                 type = EventAny.class;
                 break;
 
@@ -124,6 +129,10 @@ public abstract class Plumbing extends org.gnome.glib.Plumbing
                 type = EventProximity.class;
                 break;
 
+            case GdkEventType.OWNER_CHANGE:
+                type = EventOwnerChange.class;
+                break;
+
             case GdkEventType.WINDOW_STATE:
                 type = EventWindowState.class;
                 break;
@@ -139,12 +148,12 @@ public abstract class Plumbing extends org.gnome.glib.Plumbing
             case GdkEventType.CLIENT_EVENT:
             case GdkEventType.NO_EXPOSE:
             default:
-                throw new UnsupportedOperationException();
+                throw new UnsupportedOperationException("What GdkEventType is this?");
             }
         }
 
         proxy = org.gnome.glib.Plumbing.boxedFor(type, pointer);
-        return (Boxed) proxy;
+        return proxy;
     }
 
     private static native final int getEventTypeOrdinal(long pointer);
