@@ -1,7 +1,7 @@
 /*
  * Thing.java
  *
- * Copyright (c) 2007-2008 Operational Dynamics Consulting Pty Ltd
+ * Copyright (c) 2007-2009 Operational Dynamics Consulting Pty Ltd
  * 
  * The code in this file, and the program it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
@@ -11,6 +11,7 @@
 package com.operationaldynamics.codegen;
 
 import java.util.HashMap;
+import java.util.List;
 
 import com.operationaldynamics.driver.DefsFile;
 
@@ -56,7 +57,7 @@ public abstract class Thing
      * custom variation we have made to the .defs files; hopefully there is
      * another way we can derive this information without needing to do that).
      */
-    String importHeader;
+    List<String> importHeaders;
 
     /**
      * This is gType, of course, except for things like "const-gchar*", which
@@ -105,7 +106,7 @@ public abstract class Thing
         register(new FundamentalThing("none", "void", "void", "void"));
         register(new FundamentalThing("gchar", "char", "char", "jchar"));
         register(new FundamentalThing("guchar", "char", "char", "jchar"));
-        register(new FundamentalThing("gunichar", "char", "char", "jchar"));
+        register(new FundamentalThing("gunichar", "int", "int", "jint"));
         register(new StringThing("gchar*"));
 
         /*
@@ -170,6 +171,7 @@ public abstract class Thing
          */
         register(new FundamentalThing("int", "int", "int", "jint"));
         register(new FundamentalThing("double", "double", "double", "jdouble"));
+        register(new FundamentalArrayThing("double*", "double"));
         register(new StringThing("char*"));
         register(new FundamentalArrayThing("int*", "int"));
 
@@ -213,6 +215,11 @@ public abstract class Thing
         register(new FixmeThing("GdkAtom"));
         register(new FixmeThing("GdkAtom*"));
 
+        /*
+         * Typedefs of fundamental type
+         */
+        register(new TypedefFundamentalThing("PangoGlyphUnit", "int", "int", "jint"));
+
         /*
          * These seem to be motif-isms.
          */
@@ -231,6 +238,11 @@ public abstract class Thing
 
         register(new FixmeThing("AtkAttributeSet*"));
 
+        /*
+         * Not exposed Cairo types
+         */
+        register(new BlacklistedThing("cairo_font_options_t*"));
+
         /*
          * And what on earth are...
          */
@@ -633,8 +645,8 @@ public abstract class Thing
      * own method because only a very few object really need it, so we can use
      * the setter only in the needed Block.
      */
-    public void setImportHeader(String importHeader) {
-        this.importHeader = importHeader;
+    public void setImportHeader(List<String> importHeader) {
+        this.importHeaders = importHeader;
     }
 
     public String toString() {
