@@ -2,7 +2,7 @@
  * bindings_java_signal.c
  *
  * Copyright (c) 1998-2005 The java-gnome Team
- * Copyright (c) 2006-     Operational Dynamics Consulting Pty Ltd
+ * Copyright (c) 2006-2009 Operational Dynamics Consulting Pty Ltd
  * 
  * The code in this file, and the library it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
@@ -24,6 +24,7 @@
 #include <string.h>
 #include <glib.h>
 #include <glib-object.h>
+#include <gtk/gtk.h>
 #include "bindings_java.h"
 
 typedef struct  {
@@ -40,7 +41,7 @@ typedef struct  {
 	jchar		returnType;
  	
  	/*
- 	 * The Class upon which we invoke the [static] "handlerBlah" method.
+ 	 * The Class upon which we invoke the [static] "receiveBlah" method.
  	 */
  	jclass		receiver;
  
@@ -58,11 +59,6 @@ typedef struct  {
  	 */
  	jmethodID	method;
  	
- 	/*
- 	 * And finally, the "source" GObject that we will pass to the handler.
- 	 */
- 	GObject*	source;
- 	
 } BindingsJavaClosure;
 
  
@@ -97,15 +93,18 @@ bindings_java_marshaller
 	jboolean _b;
 	gboolean b;
 	
+	jint _e;
+	
 	jstring _str;
 	gchar* str;
+	GObject* obj;
 	
 	/*
 	 * Begin marshaller by downcasting the GClosure we got.
 	 */
 
 	bjc = (BindingsJavaClosure*) closure;
-	
+
 	/*
 	 * Get the JNIEnv interface pointer
 	 */
@@ -120,96 +119,118 @@ bindings_java_marshaller
 	 * Build the parameters for the callback. The signature of the
 	 * handlers on the Java side for a signal "name" would be:
 	 * 
-	 * 	handlerName(Signal handler, long source, type arg0, type arg1, ...)
-	 * 
+	 * 	receiveName(Signal handler, type arg0, type arg1, ...)
+	 *
+	 * Note that arg0 is universally the source object (in otherwords, a
+	 * method function where the first argument is always a reference to
+	 * self).
+	 *  
 	 * In case you didn't know, JNI's jvalue us a rather complex union
 	 * which holds any of the possible things you can send across the
 	 * boundary. So we allocate an array of them, then for each parameter
 	 * passed to the marshaller, whack them in.
 	 */
 
-	jargs = g_newa(jvalue, n_param_values + 2);
-	
+	jargs = g_newa(jvalue, n_param_values + 1);
+
 	jargs[0].l = bjc->handler;
-	jargs[1].j = (jlong) bjc->source; 
 
 	for(i = 0; i < n_param_values; i++) {
-  		type = G_VALUE_TYPE(&param_values[i]);
-    		switch(G_TYPE_FUNDAMENTAL(type)) {
+		type = G_VALUE_TYPE(&param_values[i]);
+		switch(G_TYPE_FUNDAMENTAL(type)) {
 		case G_TYPE_CHAR:
-			jargs[i+2].c = g_value_get_char(&param_values[i]);
+			jargs[i+1].c = g_value_get_char(&param_values[i]);
       			break;
 
 		case G_TYPE_UCHAR:
-			jargs[i+2].c = g_value_get_uchar(&param_values[i]);
+			jargs[i+1].c = g_value_get_uchar(&param_values[i]);
       			break;
 
 		case G_TYPE_BOOLEAN:
 			b = g_value_get_boolean(&param_values[i]);
-			jargs[i+2].z = (b == TRUE) ? JNI_TRUE : JNI_FALSE;
+			jargs[i+1].z = (b == TRUE) ? JNI_TRUE : JNI_FALSE;
 			break;
 
 		case G_TYPE_INT:
-			jargs[i+2].i = g_value_get_int(&param_values[i]);
+			jargs[i+1].i = g_value_get_int(&param_values[i]);
 			break;
 
 		case G_TYPE_UINT:
-			jargs[i+2].i = g_value_get_uint(&param_values[i]);
+			jargs[i+1].i = g_value_get_uint(&param_values[i]);
 			break;
 
 		case G_TYPE_ENUM:
-			jargs[i+2].i = g_value_get_enum(&param_values[i]);
+			jargs[i+1].i = g_value_get_enum(&param_values[i]);
 			break;
 
 		case G_TYPE_FLAGS:
-			jargs[i+2].i = g_value_get_flags(&param_values[i]);
+			jargs[i+1].i = g_value_get_flags(&param_values[i]);
 			break;
 
 		case G_TYPE_LONG:
-			jargs[i+2].j = g_value_get_long(&param_values[i]);
+			jargs[i+1].j = g_value_get_long(&param_values[i]);
 			break;
 
 		case G_TYPE_ULONG:
-			jargs[i+2].j = g_value_get_ulong(&param_values[i]);
+			jargs[i+1].j = g_value_get_ulong(&param_values[i]);
 			break;
 
 		case G_TYPE_FLOAT:
-			jargs[i+2].f = g_value_get_float(&param_values[i]);
+			jargs[i+1].f = g_value_get_float(&param_values[i]);
 			break;
 
 		case G_TYPE_DOUBLE:
-			jargs[i+2].d = g_value_get_double(&param_values[i]);
+			jargs[i+1].d = g_value_get_double(&param_values[i]);
 			break;
 
 		case G_TYPE_STRING:
-			jargs[i+2].l = (*env)->NewStringUTF(env, g_value_get_string(&param_values[i]));
+			jargs[i+1].l = bindings_java_newString(env, g_value_get_string(&param_values[i]));
       			break;
 
 		case G_TYPE_OBJECT:
+		case G_TYPE_INTERFACE:
 			/*
-			 * GObjects are just pointers, and so we pass up the address
-			 * across the boundary to be looked up and either an existing Proxy
-			 * returned or a new Proxy created. 
-			 */			
-			jargs[i+2].j = (jlong) g_value_get_object(&param_values[i]); 
+			 * GObjects are just pointers, and so we pass up the
+			 * address across the boundary to be looked up and
+			 * either an existing Proxy returned or a new Proxy
+			 * created. 
+			 */
+			obj = g_value_get_object(&param_values[i]); 
+			bindings_java_memory_cleanup(obj, FALSE);
+			jargs[i+1].j = (jlong) obj;
 			break;
 
 		case G_TYPE_BOXED:
 			/*
 			 * We make a copy of the GBoxed so that we own it and
-			 * thus it can (will) survive the duration of the signal
-			 * in the even that the developer using this code keeps a reference
-			 * to the returned Boxed.
+			 * thus it can (will) survive the duration of the
+			 * signal in the event that the developer using this
+			 * code keeps a reference to the returned Boxed.
 			 */
-			jargs[i+2].j = (jlong) g_boxed_copy(type, g_value_get_boxed(&param_values[i]));
+			jargs[i+1].j = (jlong) g_boxed_copy(type, g_value_get_boxed(&param_values[i]));
 			break;
-			
+
+		case G_TYPE_PARAM:
+			/*
+			 * We're ignoring GParamSpec at the moment. They
+			 * normally only show up in 'notify' signals, and we
+			 * don't need them. 
+			 */
+		case G_TYPE_POINTER:
+			/*
+			 * and, we're ignoring something that gets registered
+			 * as a gpointer, by definition it has no type
+			 * information and there's nothing we can do.
+			 */
+			jargs[i+1].j = (jlong) NULL;
+			break;
+
 		default:
 			/*
 			 * Unrecognized. Probably means we need to add a clause above.
 			 */
-			g_warning("Don't know how to marshal a %s", g_type_name(type));
-			jargs[i+2].l = 0;
+			g_printerr("Don't know how to marshal a %s", g_type_name(type));
+			jargs[i+1].l = 0;
 			break;
 		}
 	}
@@ -218,7 +239,7 @@ bindings_java_marshaller
 	 * And now we invoke the callback on the Java side Signal handler; we have to 
 	 * select the correct function based on what return type is necessary.
 	 */
-	
+
 	switch(bjc->returnType) {
 	case 'V':
 		/*
@@ -237,13 +258,22 @@ bindings_java_marshaller
 		} else if (_b == JNI_FALSE) {
 			b = FALSE;
 		} else {
-			g_warning("How did you manage to return a boolean that's neither TRUE nor FALSE?");
+			g_critical("How did you manage to return a boolean that's neither TRUE nor FALSE?");
 			return;
 		}
 		
 		g_value_set_boolean(return_value, b);
 		break;
+
+	case 'I':
+		/*
+		 * enum return signals
+		 */
+		_e = (*env)->CallStaticIntMethodA(env, bjc->receiver, bjc->method, jargs);		
 		
+		g_value_set_enum(return_value, _e);
+		break;
+
 	case 'L':
 		/*
 		 * String return signals
@@ -253,12 +283,12 @@ bindings_java_marshaller
 		 * enormous assumption that a string is what we get back.
 		 */
 		_str = (*env)->CallStaticObjectMethodA(env, bjc->receiver, bjc->method, jargs);
-		if (str == NULL) {
+		if (_str == NULL) {
 			g_warning("Invoking string handler returned null. That's probably bad");
-			return;
+			break;
 		}
 		
-		str = (gchar*) (*env)->GetStringUTFChars(env, _str, NULL);
+		str = (gchar*) bindings_java_getString(env, _str);
 		if (str == NULL) {
 			/* OutOfMemoryError already thrown */
 			return;
@@ -268,7 +298,7 @@ bindings_java_marshaller
 		g_value_set_string(return_value, str);
 		
 		// ... so we can release str
-		(*env)->ReleaseStringUTFChars(env, _str, str);	 
+		bindings_java_releaseString(str);	 
 		break;
 
 	default:
@@ -279,19 +309,6 @@ bindings_java_marshaller
 		break;
 	}
 
-	/*
-	 * Now, check if an exception occurred in the callback and print the
-	 * stack trace in case it did. Otherwise, the stack trace gets swallowed
-	 * (ie, we're now back at GTK which is where the code path originated.
-	 * There's nowhere else for the signal to go, is there? Well, we could
-	 * call gtk_main_quit(), but that would probably be considered rude.
-	 */
-
-	if ((*env)->ExceptionOccurred(env)) {
-		(*env)->ExceptionDescribe(env);
-		(*env)->ExceptionClear(env);
-	}
-
 	/*
 	 * Cleanup
 	 */
@@ -300,7 +317,7 @@ bindings_java_marshaller
   		type = G_VALUE_TYPE(&param_values[i]);
 		switch(G_TYPE_FUNDAMENTAL(type)) {
 		case G_TYPE_STRING:
-			(*env)->DeleteLocalRef(env, jargs[i+2].l);
+			(*env)->DeleteLocalRef(env, jargs[i+1].l);
 			break;
 
 		default:
@@ -309,8 +326,31 @@ bindings_java_marshaller
 	}
 	
 	/*
-	 * And don't need to free jargs - we alloca()'d it
+	 * Don't need to free jargs - we alloca()'d it
 	 */
+	
+	/*
+	 * Now, check if an exception occurred in the callback. There's a
+	 * catch: because we're in native code right now [care of the call to
+	 * gtk_main()] the exception gets swallowed until we return from that
+	 * native call. So we call the function which causes the main loop to
+	 * terminate, with the result that the exception will propegate out
+	 * and, yes, probably halt the program.
+	 * 
+	 * This is abrupt, but it is deliberate: we need to force developers
+	 * to deal with criticals emitted by the underlying libraries.
+	 * Otherwise, the next thing that is likely to happen is a
+	 * segmentation fault, and not only does that crash the "program" but
+	 * it takes out the Java Virtual Machine running it. People don't
+	 * like VM crashes.
+	 * 
+	 * Uncaught exceptions of any kind leaving a signal handler are to be
+	 * considered programmer error and will be fatal.
+	 */
+
+	if ((*env)->ExceptionOccurred(env)) {
+		gtk_main_quit();
+	}
 }
 
 /**
@@ -347,7 +387,6 @@ GClosure*
 bindings_java_closure_new
 (
 	JNIEnv* env, 
-	GObject* instance,
 	jobject handler,
 	jclass receiver,
 	const gchar* name,
@@ -389,6 +428,15 @@ bindings_java_closure_new
 	case G_TYPE_BOOLEAN:
 		bjc->returnType = 'Z';
       		break;
+
+	case G_TYPE_ENUM:
+		/*
+		 * If we run into a signal prototype that really does return
+		 * gint etc or a Flags this will need fixing. Until then, use
+		 * the integer type marker for enums (only).  
+		 */
+		bjc->returnType = 'I';
+      		break;
       		
 	case G_TYPE_STRING:
 		/*
@@ -401,7 +449,7 @@ bindings_java_closure_new
       	case G_TYPE_NONE:
       		bjc->returnType = 'V';
       		break;
-	
+
 	default:
 		g_critical("Don't know what to do with signal return type %s", g_type_name(info.return_type));
 		return NULL;
@@ -409,26 +457,32 @@ bindings_java_closure_new
 	
 
 	/*
-	 * the name of the methods we invoke is algorithmic: "handleName",
+	 * the name of the methods we invoke is algorithmic: "receiveName",
 	 * where Name is a PascalCase version of the signal name we were
 	 * passed in.
 	 */
-	 
-	buf = g_string_new("handle");
-	
-	gchar** tokens = g_strsplit_set(name, "_-", -1);
-	
+
+	buf = g_string_new("receive");
+
+	gchar** tokens = g_strsplit_set(name, "_-:", -1);
+
 	for (i = 0; i < g_strv_length(tokens); i++) {
 		gchar* token = tokens[i];
+
+		if (token[0] == '\0') {
+			// skip past :: which splits signal name from "detail"
+			continue;
+		}
+
 		gchar first = g_unichar_toupper(token[0]);
 		g_string_append_c(buf, first);
-		
+
 		token++; 
 		g_string_append(buf, token);
 	}
-	
+
 	methodName = buf->str;
-	
+
 	g_string_free(buf, FALSE);
 	g_strfreev(tokens);
 
@@ -487,8 +541,6 @@ bindings_java_closure_new
 	 */
 
 	bjc->handler = (*env)->NewWeakGlobalRef(env, handler);
-	
-	bjc->source = instance;
 
 	/*
 	 * And we're done!
