@@ -1,7 +1,7 @@
 /*
  * ValidateProperties.java
  *
- * Copyright (c) 2007-2008 Operational Dynamics Consulting Pty Ltd
+ * Copyright (c) 2007-2009 Operational Dynamics Consulting Pty Ltd
  * 
  * The code in this file, and the suite it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
@@ -189,7 +189,16 @@ public class ValidateProperties extends TestCaseGtk
 
         w = new Window();
         w.add(b);
-        w.showAll();
+
+        /*
+         * We call realize instead of show on the window to prevent the Window
+         * from popping onto the screen when the main loop runs in
+         * ValidateScreensAndDisplays. This is likely a weak workaround and
+         * not a fix, but we need the size-allocation to have happened to be
+         * able to complete this test fixture
+         */
+        b.showAll();
+        GtkWidget.realize(b);
 
         /*
          * Now test that the our live reference to the GtkAllocation actually
@@ -334,4 +343,20 @@ public class ValidateProperties extends TestCaseGtk
         window.add(button);
         button.grabDefault();
     }
+
+    /*
+     * Check the default visibility of Entry and then cycle the property.
+     */
+    public final void testVisibility() {
+        final Entry entry;
+
+        entry = new Entry();
+        assertTrue(entry.getVisibility());
+
+        entry.setVisibility(false);
+        assertFalse(entry.getVisibility());
+
+        entry.setVisibility(true);
+        assertTrue(entry.getVisibility());
+    }
 }
